/*===========================================================================
  SORTABLE TABLES by Inventive Labs
  * Uses Prototype 1.5+ (http://prototype.conio.net/)

  Sort a table by clicking on a column heading. To make a table sortable,
  give it a class of 'sortable':

  <table class="sortable">
    <tr>
      <th sort="integer">ID</th>
      <th>Summary</th>
      <th sort="date">Occurred on</th>
    </tr>
    <tr> (some cells...) </tr>
  </table>

  The 'sort' attribute of the table cell headers determines the way that 
  the column is sorted. The default is case-insensitive alphabetical comparison.

  Note: during the sort, every other row is given a class of 'alt' - you can
  use this to alternate background colours & etc.

  Based on tableSort.js by Inigo Surguy (http://surguy.net). This file is made 
  available under the same Creative Commons Attribution-ShareAlike 2.5 license:
  http://creativecommons.org/licenses/by-sa/2.5/
  
  **NOTE** 
  Numerous changes have been made to the original version of this file
  in order for it to work with this site's unique tables.
  Look for 'al added' or 'al changed' for unique code.
  
---------------------------------------------------------------------------*/
SortableTable = function (table) {
  var me = this; 

  this.table = table;
  //this.rows = $A(table.rows).map(function(r) { return $(r) });
  
  // all changed
  this.rows = [];
  var tmpArr = $A(table.rows);
  for (i=0; i < tmpArr.length; i++) {
  	var r = tmpArr[i];
  	if ($(r).hasClassName('nestedExpandedRow')) {
  		// attach as a sortable child of the visible row
  		if (!this.rows[this.rows.length-1].nestedExpandedRow) {
  			this.rows[this.rows.length-1].nestedExpandedRow = [];
  		}
  		
  		this.rows[this.rows.length-1].nestedExpandedRow[this.rows[this.rows.length-1].nestedExpandedRow.length] = $(r);
  	
  	} else if ($(r).hasClassName('nestedTitleRow')) {
  		// attach as a specific sub row of the visible row
  		this.rows[this.rows.length-1].nestedTitleRow = $(r);
  	
  	} else if ($(r).hasClassName('drawerRowDividerRow') || $(r).hasClassName('drawerExpandedRow')) {
  		// attach as a queued child of the visible row
  		if (!this.rows[this.rows.length-1].attachedChildren) {
  			this.rows[this.rows.length-1].attachedChildren = [];
  		}
  		
  		this.rows[this.rows.length-1].attachedChildren[this.rows[this.rows.length-1].attachedChildren.length] = $(r);
  		
  	} else {
	  	this.rows[this.rows.length] = $(r);
	}
  }
  
  this.headerRow = this.rows.shift(); 
  this.headers = Selector.findChildElements(this.headerRow, ['th']);

  this.headers.each(function(th) {
    var span = $(document.createElement('span'));
    $A(th.childNodes).each(function(c) { span.appendChild(c); });
    th.appendChild(span);
    span.onclick = function () { me.sortOnColumn(th, span) }
  });
}

SortableTable.find = function () {
  $$('table.sortable').each(function(table) { new SortableTable(table) })
}

SortableTable.prototype.simpleCompare = function(a,b) { 
  return a < b ? -1 : a == b ? 0 : 1; 
};

SortableTable.prototype.compareComposer = function(normalizeFn) { 
  var me = this;
  return function(a,b) { return me.simpleCompare(normalizeFn(a), normalizeFn(b))}
}

// Add any new comparison functions you need to this switch statement.
SortableTable.prototype.compareFunction = function (sType) { 
  switch (sType) {
    case "caseSensitive": 
      return this.simpleCompare;
    case "integer": 
      // Extracts the first numeric part of a string
      return this.compareComposer(function(a) { 
      			// al changed
      			if (a.replace(/^.*?(\d+).*?$/,"$1")) {
      				return parseInt(a.replace(/^.*?(\d+).*?$/,"$1")); 
				} else {
					return -100000;
				}
			  });
    case "float":
      // Similar, but permits floating points (.)
      return this.compareComposer(function(a) { 
      			// al changed
      			if (a.replace(/^.*?([\d\.]+).*?$/,"$1")) {
					return parseFloat(a.replace(/^.*?([\d\.]+).*?$/,"$1"));
				} else {
					return -100000.0;
				}
			  });
    case "date": 
      // Expects an ISO date format "13 MAR 2006 10:17:02 GMT"
      return this.compareComposer(Date.parse);
    // al added
    case "customDate": 
      // Expects a site unique date format "TH 2010-02-16 19:35:15"
      // 	simply trim off day of week part, and then basic string comparison
      return this.compareComposer(function(a) { return a.substring(3); });
    default:
      return this.compareComposer(function(a) {return a.toLowerCase(); });
  }
}

SortableTable.prototype.sortOnColumn = function (th, span) {

  // make sure table headers are enabled 
  if (!this.table.hasClassName('tableHeadersDisabled')) {
		
	  var table = this.table;
	  
	  
	  // figure out which column this is
	  var pos = $A(this.headerRow.cells).indexOf(th);
	  
	  // need to offset because of drawer col
	  if (table.hasClassName('nestedTable')) {
	  		pos -= 1;
	  }
	  
	  
	  // do the sort
	  var sortFn = this.compareFunction(th.getAttribute('sort'));
	  
	  // al changed
	  var order = (th.hasClassName('sortableColumnAscending') ? -1 : 1);
	  
	  
	  // al added - special case for nested row table
	  if (table.hasClassName('nestedTable')) {
	  	  
	  	  this.rows.each(
	  	  	 function(row) {
	  	  		if (row.nestedExpandedRow) {
	  	  			row.nestedExpandedRow.sort(
						function (rowA, rowB) { 
							//alert('"'+ rowA.id +'"\n"'+ rowB.id +'"\nvalid');
							//alert(order * sortFn(rowA.getCellText(pos), rowB.getCellText(pos)));
							//alert(rowA.getCellText(pos) +', '+ rowB.getCellText(pos) +'='+ sortFn(rowA.getCellText(pos), rowB.getCellText(pos)));
							return order * sortFn(rowA.getCellText(pos), rowB.getCellText(pos)); 
						}
					);
				 }
			 }
		  );
	  	
	  } else {
		  
		  this.rows.sort(
			function (rowA, rowB) { 
				// al changed
				if (!rowA.visible() || !rowB.visible()
						|| rowA.hasClassName('tableTopRow') || rowA.hasClassName('tableBottomRow') 
						|| rowB.hasClassName('tableTopRow') || rowB.hasClassName('tableBottomRow')) {
					//alert('"'+ rowA.id +'"\n"'+ rowB.id +'"\nskip');
					return 0;
				} else {
					//alert('"'+ rowA.id +'"\n"'+ rowB.id +'"\nvalid');
					//alert(order * sortFn(rowA.getCellText(pos), rowB.getCellText(pos)));
					return order * sortFn(rowA.getCellText(pos), rowB.getCellText(pos)); 
				}
			}
		  );
	  
	  }
	  
	  // al added
	  this.headers.each(function(tmp) {
			tmp.removeClassName('sortableColumnDescending');
			tmp.removeClassName('sortableColumnAscending');
		});
	  if (order == 1) {
		th.addClassName('sortableColumnAscending');
	  } else {
		th.addClassName('sortableColumnDescending');
	  }
	
	
	  // rearrange the rows based on sort results
	  var alt = 0;
	  var altNested = 0;
	  var absCount = 0;
	  var rowCount = this.rows.length;
	  var tbody = this.table.tBodies[0];
	  this.rows.each(function(row) { 
	  	
	  	// al changed
		if (!table.hasClassName('drawerTable') && row.visible()
				&& !row.hasClassName('tableTopRow') && !row.hasClassName('tableBottomRow')) {
			row.removeClassName('oddRow');
			row.removeClassName('evenRow');
			
			if (alt % 2) { 
			  row.addClassName('evenRow'); 
			} else {
			  row.addClassName('oddRow');
			}
			
			alt += 1;
		}
		tbody.appendChild(row);
		
		// al added - special case for nested row table
	  	if (row.nestedTitleRow) {
			tbody.appendChild(row.nestedTitleRow);
		}
		
		// al added - another, special case for nested row table
		if (row.nestedExpandedRow) {
			altNested = 1;
			row.nestedExpandedRow.each(function(nestedRow) {
							nestedRow.removeClassName('oddRow');
							nestedRow.removeClassName('evenRow');
			
							if (altNested % 2) { 
			 					nestedRow.addClassName('evenRow'); 
							} else {
							  	nestedRow.addClassName('oddRow');
							}
			
							altNested += 1;
							
							tbody.appendChild(nestedRow);
						}
			 		);
		}
		
		// al added - also append row's children, if exist
		if (row.attachedChildren) {
			row.attachedChildren.each(function(child) { 
					tbody.appendChild(child);
					
					if (child.hasClassName('drawerRowDividerRow')) {
						if (absCount == (rowCount-2)) {
							child.hide();
						} else {
							child.show();
						}
					}
				});
		}
		
		absCount += 1;
		
	  });
  
  }
}

Element.addMethods({
  getText:function(e){return e.text = e.text||e.textContent||e.innerText||''},
  getCellText: function(row, pos) { 
  	// al changed
  	if (row.down("td", pos)) {
    	row.cellTexts = row.cellTexts || [];
    	row.cellTexts[pos] = row.cellTexts[pos] || row.down("td", pos).getText(); 
    	return row.cellTexts[pos];
    }
  }
});

Event.observe(window, 'load', SortableTable.find, false);
